#pragma once

#include "CommonType_Private.h"
#include "Observer.h"
#include "Singleton.h"

#define SAFE_DELETE(x)	\
	if(x){				\
		delete x;		\
		x = NULL;		\
	}

class CMainActor;
class CActor;
class CManager;

class _IGTAPIEnvironmentEvent
{
public:
	virtual ~_IGTAPIEnvironmentEvent(void) { }
	
	virtual void OnGTAPIEnvironmentEvent_Update(void) = 0;
};


class _IGTActorManagementEvent
{
public:
	virtual ~_IGTActorManagementEvent(void) { }

	virtual void OnGTManagementEvent_DeleteActor(CActor *pActor) { }
	virtual void OnGTManagementEvent_DeleteManager(CManager *pManager) { }
};


#define GT_GLOBAL	CSingleton<CGameTalkAPIGlobal>::Instance()

class CMessengerActor;

class CGameTalkAPIGlobal
{
public:
	CGameTalkAPIGlobal(void);
	~CGameTalkAPIGlobal(void);

	void Initialize(void)
	{
		m_bGameTalkApiStarted = true;
	}

	void Finalize(void)
	{
		m_bGameTalkApiStarted = false;
	}

	trid_t GetTransactionID(trid_t *pTrid);

	bool IsEnvironmentSettingOK(void)
	{
		return m_bGameTalkApiEnvSet;
	}

	bool IsAvailableAPI(void)
	{
		return m_bGameTalkApiStarted;
	}

	void SetEnvironment(const GT_API_ENVIRONMENT_DATA &sEnv);

	P_GT_API_ENVIRONMENT_DATA & GetEnvironment(void)
	{
		return m_sEnvironmentData;
	}

	P_GT_INTERNAL_ENV_DATA & GetInternalEnvironment(void)
	{
		return m_sInternalEnvData;
	}

	CGTObserved<_IGTAPIEnvironmentEvent> & GetAPIEnvironmentObserver(void)
	{
		return m_apiEnvObserver;
	}

	P_GT_VOICE_ROOM_USER_DATA & GetNullRoomUserData(void)
	{
		return m_sNullRoomUserData;
	}

	CMessengerActor * GetMessengerActor(int nUserID);
	CMessengerActor * CreateMessengerActor(int nUserID);
	bool DeleteMessengerActor(int nUserID);

private:
	void _LoadInternalEnv(void);
	void _VerifyEnvironmentData(void);

public:
	// ޽ Actor  
	typedef std::map<int, CMessengerActor *> messengeractormap_t;
	messengeractormap_t m_mapMessengerActor;

	// ȯ  
	CGTObserved<_IGTAPIEnvironmentEvent>	m_apiEnvObserver;
	P_GT_API_ENVIRONMENT_DATA m_sEnvironmentData;
	P_GT_INTERNAL_ENV_DATA m_sInternalEnvData;

	bool m_bGameTalkApiEnvSet;
	bool m_bGameTalkApiStarted;

	P_GT_VOICE_ROOM_USER_DATA m_sNullRoomUserData;    
};

